using System;
using System.Linq;
using ClientServer.Db;
using ClientServer.Models.Exercises;
using ClientServer.Models.Exercises.AfterSolutions;
using ClientServer.Models.Exercises.Solution;

namespace ClientServer.Helpers
{
    /// <summary>
    /// to enable editing readonly template files after release make sure for all solution get methods we use
    /// the file content from the code template of the exercise
    /// </summary>
    public static class UserSolutionHelper
    {
        /// <summary>
        /// replaces all readonly file content (an name) with the contents from the coe template file
        /// also replaces hidden files with the real content
        /// </summary>
        /// <param name="solutionWithFiles">the solution with template files loaded</param>
        /// <param name="codeTemplate">the correct code template (same plang id as solution) with loaded template files</param>
        /// <param name="applyTemplateFileVisibilityResetToEmpty">true: to apply template content visibility and set the content to the template string</param>
        /// <param name="applyTemplateFileVisibilityResetToTemplateContent">true: to apply template content and reset the content to the template content
        ///   (used for e.g. testing because then we actually need the content)
        ///   (this has higher priority than applyTemplateFileVisibilityResetToEmpty!)</param>
        /// <returns></returns>
        public static string ReplaceReadonlyFiles(Solution solutionWithFiles, CodeTemplate codeTemplate,
            bool applyTemplateFileVisibilityResetToEmpty, bool applyTemplateFileVisibilityResetToTemplateContent)
        {
            if (solutionWithFiles.PLangId != codeTemplate.PLangId)
            {
                var errorMsg = $"solution plang id ({solutionWithFiles.PLangId}) and template plang id ({codeTemplate.PLangId}) were not equal.";
                Console.WriteLine($"[WARNING] {errorMsg}");
                return errorMsg;

            }

            foreach (var solutionFile in solutionWithFiles.SolutionFiles)
            {
                if (solutionFile.TemplateFileId.HasValue)
                {
                    var templateFile =
                        codeTemplate.TemplateFiles.FirstOrDefault(p => p.Id == solutionFile.TemplateFileId);


                    if (templateFile == null)
                    {
                        var errorMsg = $"could not find code template file with id {solutionFile.TemplateFileId} " +
                                        $"for solution with userId: {solutionWithFiles.UserId}, releaseId: {solutionWithFiles.ExerciseReleaseId}, plangId: {solutionWithFiles.PLangId}";
                        
                        Console.WriteLine($"[WARNING] {errorMsg}");
                        return errorMsg;
                    }

                    //we need this for testing because the user could have an old (own) version of the file
                    //because we changed from visible to hidden (and the user changed the file)
                    //for testing we want to use the template content (current version) not the old user version
                    if (applyTemplateFileVisibilityResetToTemplateContent &&
                        templateFile.IsContentVisibleForUser == false)
                    {
                        solutionFile.Content = templateFile.Content;
                        solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                        continue;
                    }
                    
                    if (applyTemplateFileVisibilityResetToEmpty && templateFile.IsContentVisibleForUser == false)
                    {
                        solutionFile.Content = "";
                        solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                        continue;
                    }
                    
                    if (templateFile.IsEditableByUser) continue;
                    //else replace content

                    solutionFile.Content = templateFile.Content;
                    solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                }
            }


            return null;
        }

        /// <summary>
        /// same for after solution    
        /// </summary>
        /// <param name="solutionWithFiles"></param>
        /// <param name="codeTemplate"></param>
        /// <param name="applyTemplateFileVisibilityResetToEmpty">true: to apply template content visibility and set the content to the template string</param>
        /// <param name="applyTemplateFileVisibilityResetToTemplateContent">true: to apply template content and reset the content to the template content
        ///   (used for e.g. testing because then we actually need the content)
        ///   (this has higher priority than applyTemplateFileVisibilityResetToEmpty!)</param>
        /// <returns></returns>
        public static string ReplaceReadonlyFiles(AfterSolution solutionWithFiles, CodeTemplate codeTemplate,
            bool applyTemplateFileVisibilityResetToEmpty, bool applyTemplateFileVisibilityResetToTemplateContent)
        {
            if (solutionWithFiles.SolutionPLangId != codeTemplate.PLangId)
            {
                var errorMsg = $"after solution plang id ({solutionWithFiles.SolutionPLangId}) and template plang id ({codeTemplate.PLangId}) were not equal.";
                Console.WriteLine($"[WARNING] {errorMsg}");
                return errorMsg;

            }

            foreach (var solutionFile in solutionWithFiles.SolutionFiles)
            {
                if (solutionFile.TemplateFileId.HasValue)
                {
                    var templateFile =
                        codeTemplate.TemplateFiles.FirstOrDefault(p => p.Id == solutionFile.TemplateFileId);


                    if (templateFile == null)
                    {
                        var errorMsg = $"could not find code template file with id {solutionFile.TemplateFileId} " +
                                       $"for solution with userId: {solutionWithFiles.SolutionUserId}, releaseId: {solutionWithFiles.SolutionExerciseReleaseId}, plangId: {solutionWithFiles.SolutionPLangId}";
                        Console.WriteLine($"[WARNING] {errorMsg}");
                        return errorMsg;
                    }

                    //we need this for testing because the user could have an old (own) version of the file
                    //because we changed from visible to hidden (and the user changed the file)
                    //for testing we want to use the template content now not the old user version
                    if (applyTemplateFileVisibilityResetToTemplateContent &&
                        templateFile.IsContentVisibleForUser == false)
                    {
                        solutionFile.Content = templateFile.Content;
                        solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                        continue;
                    }
                    
                    if (applyTemplateFileVisibilityResetToEmpty && templateFile.IsContentVisibleForUser == false)
                    {
                        solutionFile.Content = "";
                        solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                        continue;
                    }


                    if (templateFile.IsEditableByUser) continue;
                    //else replace content

                    solutionFile.Content = templateFile.Content;
                    solutionFile.FileNameWithExtension = templateFile.FileNameWithExtension;
                }
            }


            return null;
        }
    }
}
